"""
    Sample metadata API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 7.14.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from metamist.api_client import ApiClient, Endpoint as _Endpoint
from metamist.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types,
    async_wrap
)
from metamist.model.body_get_project_insights_details import BodyGetProjectInsightsDetails
from metamist.model.body_get_project_insights_summary import BodyGetProjectInsightsSummary
from metamist.model.http_validation_error import HTTPValidationError
from metamist.model.project_insights_details import ProjectInsightsDetails
from metamist.model.project_insights_summary import ProjectInsightsSummary


class ProjectInsightsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self.get_project_insights_details_async = async_wrap(self.get_project_insights_details)
        self.get_project_insights_details_endpoint = _Endpoint(
            settings={
                'response_type': ([ProjectInsightsDetails],),
                'auth': [
                    'HTTPBearer'
                ],
                'endpoint_path': '/api/v1/project-insights/project-insights-details',
                'operation_id': 'get_project_insights_details',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'body_get_project_insights_details',
                ],
                'required': [
                    'body_get_project_insights_details',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'body_get_project_insights_details':
                        (BodyGetProjectInsightsDetails,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'body_get_project_insights_details': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )

        self.get_project_insights_summary_async = async_wrap(self.get_project_insights_summary)
        self.get_project_insights_summary_endpoint = _Endpoint(
            settings={
                'response_type': ([ProjectInsightsSummary],),
                'auth': [
                    'HTTPBearer'
                ],
                'endpoint_path': '/api/v1/project-insights/project-insights-summary',
                'operation_id': 'get_project_insights_summary',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'body_get_project_insights_summary',
                ],
                'required': [
                    'body_get_project_insights_summary',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'body_get_project_insights_summary':
                        (BodyGetProjectInsightsSummary,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'body_get_project_insights_summary': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client
        )

    def get_project_insights_details(
        self,
        body_get_project_insights_details,
        **kwargs
    ):
        """Get Project Insights Details  # noqa: E501

        Get extensive sequencing group details for a list of projects and sequencing types  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_project_insights_details(body_get_project_insights_details, async_req=True)
        >>> result = thread.get()

        Args:
            body_get_project_insights_details (BodyGetProjectInsightsDetails):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            [ProjectInsightsDetails]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['body_get_project_insights_details'] = \
            body_get_project_insights_details
        return self.get_project_insights_details_endpoint.call_with_http_info(**kwargs)

    def get_project_insights_summary(
        self,
        body_get_project_insights_summary,
        **kwargs
    ):
        """Get Project Insights Summary  # noqa: E501

        Get summary stats for a list of projects and sequencing types  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_project_insights_summary(body_get_project_insights_summary, async_req=True)
        >>> result = thread.get()

        Args:
            body_get_project_insights_summary (BodyGetProjectInsightsSummary):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            [ProjectInsightsSummary]
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['body_get_project_insights_summary'] = \
            body_get_project_insights_summary
        return self.get_project_insights_summary_endpoint.call_with_http_info(**kwargs)

